@@ -67,7 +67,7 @@ public:
 };
 #endif
 
-static bool computeAutoSync(KProperty *property, bool defaultAutoSync)
+static bool computeAutoSync(const KProperty *property, bool defaultAutoSync)
 {
     return (property->autoSync() != 0 && property->autoSync() != 1) ?
                 defaultAutoSync : (property->autoSync() != 0);
@@ -103,7 +103,7 @@ static int getIconSize(int fontPixelSize)
     return fontPixelSize * 0.85;
 }
 
-static int typeForProperty( KProperty* prop )
+static int typeForProperty(const KProperty* prop)
 {
     if (prop->listData())
         return KProperty::ValueFromList;
@@ -120,9 +120,9 @@ void ItemDelegate::paint(QPainter *painter,
     if (gridLineColor.isValid()) {
         alteredOption.rect.setTop(alteredOption.rect.top() + 1);
     }
-    painter->save();
+    const KPropertyUtils::PainterSaver saver(painter);
     QRect r(option.rect);
-    const KPropertyEditorDataModel *editorModel = dynamic_cast<const KPropertyEditorDataModel*>(index.model());
+    const KPropertyEditorDataModel *editorModel = qobject_cast<const KPropertyEditorDataModel*>(index.model());
     if (!editorModel) {
         return;
     }
@@ -194,7 +194,6 @@ void ItemDelegate::paint(QPainter *painter,
         painter->drawLine(r.topLeft(), r.topRight());
     }
     //kprDebug()<<"rect:" << r << "viewport:" << painter->viewport() << "window:"<<painter->window();
-    painter->restore();
 }
 
 QSize ItemDelegate::sizeHint(const QStyleOptionViewItem &option,
@@ -208,9 +207,12 @@ QWidget * ItemDelegate::createEditor(QWidget * parent,
 {
     if (!index.isValid())
         return 0;
-    QStyleOptionViewItem alteredOption(option);
-    KProperty *property = KPropertyUtils::propertyForIndex(index);
+    const KProperty *property = KPropertyUtils::propertyForIndex(index);
+    if (property && property->isReadOnly()) {
+        return nullptr;
+    }
     const int t = property ? typeForProperty(property) : KProperty::String;
+    QStyleOptionViewItem alteredOption(option);
     alteredOption.rect.setHeight(alteredOption.rect.height()+3);
     QWidget *w = KPropertyWidgetsPluginManager::self()->createEditor(t, parent, alteredOption, index);
     if (!w) {
@@ -331,7 +333,7 @@ void KPropertyEditorView::changeSetInternal(KPropertySet *set, SetOptions option
         if (index.isValid()) {
 //! @todo This crashes when changing the interpreter type in the script plugin
 #if 0
-            Property *property = d->model->propertyForIndex(index);
+            KProperty *property = d->model->propertyForIndex(index);
             //if (property->isNull())
             //    kprDebug() << "WTF? a NULL property?";
             //else
@@ -369,6 +371,8 @@ void KPropertyEditorView::changeSetInternal(KPropertySet *set, SetOptions option
                 this, SLOT(slotPropertyReset(KPropertySet&,KProperty&)));
         connect(d->set, SIGNAL(aboutToBeCleared()), this, SLOT(slotSetWillBeCleared()));
         connect(d->set, SIGNAL(aboutToBeDeleted()), this, SLOT(slotSetWillBeDeleted()));
+        connect(d->set, &KPropertySet::readOnlyFlagChanged,
+                this, &KPropertyEditorView::slotReadOnlyFlagChanged);
     }
 
     KPropertyEditorDataModel *oldModel = d->model;
@@ -412,6 +416,16 @@ void KPropertyEditorView::slotSetWillBeDeleted()
     changeSet(0, QByteArray());
 }
 
+void KPropertyEditorView::slotReadOnlyFlagChanged()
+{
+    const QModelIndex index = currentIndex();
+    setCurrentIndex(QModelIndex());
+    if (index.isValid()) {
+        selectionModel()->select(index, QItemSelectionModel::Select);
+        setCurrentIndex(index);
+    }
+}
+
 void KPropertyEditorView::setAutoSync(bool enable)
 {
     d->autoSync = enable;
@@ -429,9 +443,14 @@ void KPropertyEditorView::currentChanged( const QModelIndex & current, const QMo
 
 bool KPropertyEditorView::edit( const QModelIndex & index, EditTrigger trigger, QEvent * event )
 {
-    bool result = QTreeView::edit( index, trigger, event );
+    bool result;
+    if (!d->set || d->set->isReadOnly()) {
+        result = false;
+    } else {
+        result = QTreeView::edit(index, trigger, event);
+    }
     if (result) {
-      QLineEdit *lineEditEditor = dynamic_cast<QLineEdit*>( (QObject*)d->itemDelegate->m_currentEditor );
+      QLineEdit *lineEditEditor = qobject_cast<QLineEdit*>(d->itemDelegate->m_currentEditor.data());
       if (lineEditEditor) {
         lineEditEditor->deselect();
         lineEditEditor->end(false);
@@ -456,10 +475,8 @@ QRect KPropertyEditorView::revertButtonArea( const QModelIndex& index ) const
     int x2 = columnWidth(0);
     int x1 = x2 - iconSize - 2;
     QRect r(visualRect(index));
-//    kprDebug() << r;
     r.setLeft(x1);
     r.setRight(x2);
-//    kprDebug() << r;
     return r;
 }
 
